# -*- coding: utf-8 -*-
from identity_client import PERSISTENCE_MODULE

Identity = PERSISTENCE_MODULE.models.Identity
ServiceAccount = PERSISTENCE_MODULE.models.ServiceAccount
AccountMember = PERSISTENCE_MODULE.models.AccountMember
